/*
 * DataLogger_GUI.pde 
 * 
 * Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the project's author nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/******************************************************************************
 *       MSP-EXP430FR5739 and CC3000 Data Logger Example GUI
 * Desc:
 * This PC GUI application communicates with the MSP-EXP430FR5739 via the CC3000  
 * module to send receive regular temperature updates and send temperature 
 * setpoints and dimmer values for 2 LEDs. 
 * 
 * to communicate to the CC3000 we use 2 UDP sockets and use multicast addresses
 * We use multicast because this application dosen't know what the IP address of 
 * the CC3000 nor does it know ours.  Furthermore, multicast is useful if we wanted 
 * multiple listeners.  One socket is used for receiving the logged temperature 
 * data the other socket is used to receive the accelerometer data.  This application
 * doesn't sendany data as the CC3000 is turned on only when it is sending data
 * to conserve power and therefore isn't listening.  If needed one could leave the 
 * CC3000 on for a period after sending data to the Server and then the server could 
 * send data. 
 * 
 *
 * M. Schneider
 * PDE, Ltd.
 * Ver 1.00 September 2011
 ******************************************************************************/ 
// import libraries
// UDP Library
import hypermedia.net.*;
// OPENGL library
import processing.opengl.*;
// animation library
import de.looksgood.ani.*;

/*
* UDP multicast address and port definitions
*/
UDP udpTemperature, udpAccelerometer;  
String MULTICAST_IP       = "224.0.0.1";	
int TEMPERATURE_PORT      = 0xC748;
int ACCELEROMETER_PORT    = 0xC749;

/*
* main objects
*/
classThermostat Thermostat;
classAccelerometer Accel;
classGraph ShortTermGraph, LongTermGraph;
classPowerGraph PowerGraph;

/*
* Temperature Data Log Buffers
*/
float[] ShortTermTimeAlignedBuffer = new float[60];
float[] LongTermTimeAlignedBuffer = new float[1440];
/*
* MSP430 and CC3000 Tx Power Estimate Buffer
*/
float[] MspPowerData = new float[150];
int MspPowerActiveIndex = 0;

/*
* misc
*/
String TI_LINK = "http://www.ti.com"; 
String PDE_LINK = "http://www.pde.com"; 
PImage PDEImg, TIImg, UpHotSpot, DownHotSpot;
int TI_IMG_X = 840; 
int TI_IMG_Y = 573;
int PDE_IMG_X = 29;
int PDE_IMG_Y = 612;
PFont f;
String TemperatureUnitsString;
int AccelerometerLastUpdateMs = -1001;
int lastSecond;
int NumOfBytesReceivedCounter = 0;
boolean HotSpotActive = false;

/*
* defines view. The GUI only displays one view at a time.
* there are 4 views which show either the Temperature
* short-term log, long-term log, Power or the accerometer data.
* Two toggle between the short-term, long-term and power  views use the 
* mouse to hover over the up and down arrow view at the top
* and bottom of the GUI.  the Acceleromter data view is received
* automatically displayed when there is accelereomter data.
* Once the accelerometer data stops for 1 second the last view is restored
*/
int View, NewView, LastView;
int SHORT_TERM_GRAPH_VIEW =  0;
int LONG_TERM_GRAPH_VIEW =   1;
int POWER_GRAPH_VIEW =       2;
int ACCELEROMETER_VIEW =     3;

/**
 *  Define the GUI layout
 */ 
 // Thermostat
int THERM_X_LOC  = 75;
int THERM_Y_LOC  = 150;
// Short-Term Graph starting location
int SHORT_TERM_X_LOC = 300;
int SHORT_TERM_Y_LOC = (681/2)-100;
// Long-Term Graph starting location
int LONG_TERM_X_LOC = 300;
int LONG_TERM_Y_LOC = (681/2) - 100;
// Power Graph starting location
int POWER_GRAPH_X_LOC   = 300;
int POWER_GRAPH_Y_LOC = (681/2);
// Accelerometer starting location
int ACCELEROMETER_X_LOC = (952/2) ;
int ACCELEROMETER_Y_LOC = (681/2) - 100;


// Variables used for animating when changing views
int ShortTermGraphYoffset = -500;
int LongTermGraphYoffset  = LONG_TERM_Y_LOC;
int PowerGraphYoffset     = -500;
int AccelerometerYoffset  = -500;
    
String[] ViewArray = {"ShortTermGraphYoffset", "LongTermGraphYoffset", "PowerGraphYoffset"};
int[] ViewYOffsetArray = {SHORT_TERM_Y_LOC, LONG_TERM_Y_LOC, POWER_GRAPH_Y_LOC};
/**
 * init
 */
void setup() {  
  size(952, 681, OPENGL); 
  smooth();
  f = loadFont("ComicSansMS-24.vlw"); 
  textFont(f); 
  Ani.init(this);
  
  // Open Configuration file
  ReadConfigFile();
  
  // create a new datagram connection on ports TEMPERATURE_PORT and ACCELEROMETER_PORT
  // and wait for incomming message  
  udpTemperature = new UDP( this, TEMPERATURE_PORT, MULTICAST_IP);  
  //udpTemperature.log( true ); 		// <-- printout the connection activity  
  udpTemperature.setReceiveHandler("TemperatureRxHandler");
  udpTemperature.listen( true );    
  
  udpAccelerometer = new UDP( this, ACCELEROMETER_PORT, MULTICAST_IP);  
  //udpAccelerometer.log( true ); 		// <-- printout the connection activity  
  udpAccelerometer.setReceiveHandler("AccelerometerRxHandler");
  udpAccelerometer.listen( true );  
  
  println(udpTemperature.isJoined());
  println(udpTemperature.isMulticast());
  
  // Setup the Thermostat
  Thermostat = new classThermostat(THERM_X_LOC, THERM_Y_LOC);  
  // Set Units for Thermostat and graph
  if(TemperatureUnitsString.equals("C"))
  {
    Thermostat.SetUnits(Thermostat.UNITS_C);
  }
  else if(TemperatureUnitsString.equals("F"))
  {
    Thermostat.SetUnits(Thermostat.UNITS_F);
  }  
  Thermostat.setup();
  
  // Load Acceleromter
  Accel = new classAccelerometer();
  Accel.setup();
  
  // Navigation HotSpots
  DownHotSpot = loadImage("DownHotSpot2.png");
  UpHotSpot = loadImage("UpHotSpot2.png");
  
  // Logos
  PDEImg = loadImage("PDE_Logo.jpg");
  TIImg = loadImage("TI_Logo.jpg");
    
  // Load Graphs
  ShortTermGraph = new classGraph(0,-500, 60, 8);
  LongTermGraph = new classGraph(0,0, 1440,0.33333333333333);
  ShortTermGraph.setup();
  LongTermGraph.setup();
  PowerGraph = new classPowerGraph((width-MspPowerData.length)-5,50+5);
  PowerGraph.setup();
  
  // Set Units for the graph
  if(TemperatureUnitsString.equals("C"))
  {
    ShortTermGraph.SetUnits(Thermostat.UNITS_C);
    LongTermGraph.SetUnits(Thermostat.UNITS_C);
  }
  else if(TemperatureUnitsString.equals("F"))
  {
    ShortTermGraph.SetUnits(Thermostat.UNITS_F);
    LongTermGraph.SetUnits(Thermostat.UNITS_F);
  }
  
  // Set Defaults
  View = LastView = NewView = LONG_TERM_GRAPH_VIEW;
  Ani.to(this, .1, "ShortTermGraphYoffset", -500);
  Ani.to(this, 5, "LongTermGraphYoffset", LONG_TERM_Y_LOC);
  
  // Sync Time
  lastSecond = second();
  
}

//process events
void draw() {
  // Refresh screen
  background(0x262626);
    
  // Calculate average MSP + CC3000 Power
  if(second() != lastSecond)
  {
    lastSecond = second();
    // How Many bytes did we receive over the last second?
    int NumOfBytesReceivedPerSeconds = NumOfBytesReceivedCounter;
    // Reset counter for next second
    NumOfBytesReceivedCounter = 0;
    //
    // Assuming a fixed Data Rate of 7Mbps we calculate the estimated Tx
    // time.
    float TxTime = float(NumOfBytesReceivedPerSeconds) / 7000000.0;    
    //
    // Average Tx Power is 180mA * 3.3V = 594mW.
    // Calcuate Tx Power based on this assumption and estimate
    float TxPower = TxTime * 594.0;
    
    // insert to array
    if(MspPowerActiveIndex >= MspPowerData.length)
    {
      // Buffer is full, pop one off the end
      arraycopy(MspPowerData, 1, MspPowerData, 0, MspPowerData.length-1);
      MspPowerData[MspPowerData.length-1] = TxPower;
    }
    else
    {
      MspPowerData[MspPowerActiveIndex] = TxPower;
      MspPowerActiveIndex++;
    }
  }
  
  // Draw Thermometer
  Thermostat.draw();
  
  // Determine Active View
  UpdateViewAndDrawHotSpots();
  
  Accel.draw(ACCELEROMETER_X_LOC, AccelerometerYoffset);
  ShortTermGraph.draw(ShortTermTimeAlignedBuffer, SHORT_TERM_X_LOC, ShortTermGraphYoffset);
  LongTermGraph.draw(LongTermTimeAlignedBuffer, LONG_TERM_X_LOC, LongTermGraphYoffset);
    
  PowerGraph.draw(MspPowerData, MspPowerActiveIndex, POWER_GRAPH_X_LOC, PowerGraphYoffset);
  // Draw Logos
  DrawLogos();
}

void UpdateViewAndDrawHotSpots(){
  // Draw Up and Down Hot Spots
  image(UpHotSpot, width/2 - UpHotSpot.width/2 , 0); 
  image(DownHotSpot, width/2 - DownHotSpot.width/2, height-(DownHotSpot.height)); 
  // Only display the temperature graphs if the accelerometer
  // view is not activated
  int Direction = 0;
  if( (mouseY < (UpHotSpot.height)) &&
      (mouseX > (width/2 - UpHotSpot.width/2 )) &&
      (mouseX < (width/2 + UpHotSpot.width/2 )) )
  {          
    if(HotSpotActive == false)
    {
      HotSpotActive = true;
      Direction = 1;
    }
  }
  else if( (mouseY > (height - DownHotSpot.height)) &&
           (mouseX > (width/2 - DownHotSpot.width/2 )) &&
           (mouseX < (width/2 + DownHotSpot.width/2 )) )
  {
    if(HotSpotActive == false)
    {      
      HotSpotActive = true;  
      Direction = -1;
    }
  }
  else
  {
    HotSpotActive = false;
  }
  
    // if last update was less than 2 seconds ago then display the accelerometer view
  if(millis() < (AccelerometerLastUpdateMs + 1000))
  {
    NewView = ACCELEROMETER_VIEW;
    // See if view was just updated
    if(View != NewView)
    {
      // Transition Up
      Ani.to(this, 5, ViewArray[LastView], -500);
      Ani.to(this, 0, "AccelerometerYoffset", height + 500);
      Ani.to(this, 5, "AccelerometerYoffset", ACCELEROMETER_Y_LOC); 
      // backup the old view
      LastView = View;   
    }
    View = NewView;
  }
  else
  {
    if(View == ACCELEROMETER_VIEW)
    {
      Ani.to(this, 5,"AccelerometerYoffset", -500);
      View = LastView - 1;
      Direction = 1;
    }
    if(Direction != 0)
    {
      View += Direction;
      if(View < 0)
      {
        View = 2;
      }      
      View %= 3;
      
      println(ViewArray[LastView] + "," + ViewArray[View]);
      if(Direction > 0)
      {
        Ani.to(this, 0, ViewArray[View], height + 500);
        Ani.to(this, 5, ViewArray[LastView], -500);
      }
      else
      {
        Ani.to(this, 0, ViewArray[View], -500);
        Ani.to(this, 5, ViewArray[LastView], height + 500);
      }
      Ani.to(this, 5, ViewArray[View], ViewYOffsetArray[View]);       
      LastView = View;
            
    }
  }
}
/*void UpdateViewAndDrawHotSpots() {
  // Draw Up and Down Hot Spots
  image(UpHotSpot, width/2 - UpHotSpot.width/2 , 0); 
  image(DownHotSpot, width/2 - DownHotSpot.width/2, height-(DownHotSpot.height)); 
  
  // if last update was less than 2 seconds ago then display the accelerometer view
  if(millis() < (AccelerometerLastUpdateMs + 1000))
  {
    NewView = ACCELEROMETER_VIEW;
    // See if view was just updated
    if(View != NewView)
    {
      // backup the old view
      LastView = View;      
      if(LastView == SHORT_TERM_GRAPH_VIEW)
      {
        // Transition Up
        Ani.to(this, 5, "ShortTermGraphYoffset", -500);
        Ani.to(this, 5, "AccelerometerYoffset", ACCELEROMETER_Y_LOC); 
      }
      else
      {
        // Transition Down
        Ani.to(this, 5, "LongTermGraphYoffset", height + 500);
        Ani.to(this, 5, "AccelerometerYoffset", ACCELEROMETER_Y_LOC);        
      }      
    }
    View = NewView;
  }
  else
  {
    // Check to see if we are returning from the Accel View
    if(View == ACCELEROMETER_VIEW)
    {
      // retrieve last view
      NewView = LastView;
    }
    // Only display the temperature graphs if the accelerometer
    // view is not activated
    if( (mouseY < (UpHotSpot.height)) &&
        (mouseX > (width/2 - UpHotSpot.width/2 )) &&
        (mouseX < (width/2 + UpHotSpot.width/2 )) )
    {          
      if(HotSpotActive == false)
      {
        HotSpotActive = true;
        NewView = View + 1;
        NewView %= 3;
      }
      //LONG_TERM_GRAPH_VIEW;
    }
    else if( (mouseY > (height - DownHotSpot.height)) &&
             (mouseX > (width/2 - DownHotSpot.width/2 )) &&
             (mouseX < (width/2 + DownHotSpot.width/2 )) )
    {
      if(HotSpotActive == false)
      {
        //NewView = SHORT_TERM_GRAPH_VIEW;  
        NewView = View - 1;
        if(NewView == -1)
        {
          NewView = 2;
        }
        HotSpotActive = true;  
      }
    }
    else
    {
      HotSpotActive = false;
    }
    // See if view was just updated
    if(View != NewView)
    {      
      int DeltaView;
      // Find Direction
      DeltaView = NewView - View;
      if(DeltaView > 0)
      {
        // Transition Up                
        if(View == ACCELEROMETER_VIEW)
        {
          Ani.to(this, 5, "AccelerometerYoffset", -500);
        }
        else
        {
          Ani.to(this, 5, ViewArray[View], -500);
        }        
        Ani.to(this, 5, ViewArray[NewView], LONG_TERM_Y_LOC);        
      }
      else
      {
        // Transition Down        
        if(View == ACCELEROMETER_VIEW)
        {
          Ani.to(this, 5, "AccelerometerYoffset", height + 500);
        }
        else
        {
          Ani.to(this, 5, ViewArray[View], height + 500);
        }        
        Ani.to(this, 5, ViewArray[NewView], SHORT_TERM_Y_LOC);
      }
      
      // Update View
      LastView = View = NewView;
    }
  }
    
}*/

/**
 * To perform any action on datagram reception, you need to implement this 
 * handler in your code. This method will be automatically called by the UDP 
 * object each time he receive a non-null message.
 * By default, this method have just one argument (the received message as 
 * byte[] array), but in addition, two arguments (representing in order the 
 * sender IP address and his port) can be set like below.
 *
 * Receives either the Long Term or the short Term log of temperature from
 * the MSP430 and CC3000.  The Long Term (24Hrs) contains 1440 bytes and
 * the Short Term (60s) contains 60 bytes of temperature data.  Therefore,
 * the data length can be used to determine the buffer type. Both buffers 
 * are copied to local buffers. If the packet is a short-term packet then
 * the current temperature is also updated. 
 */
void TemperatureRxHandler( byte[] data, String ip, int port ) {	// <-- extended handler
  NumOfBytesReceivedCounter += data.length;
  byte[] ByteDataBuffer = new byte[1440];
  int DataLength = int(data[1]<<8) + int(data[0]);
  int ZeroOffset = int(data[3]<<8) + int(data[2]);
  
  // use the length to determine if this is a Long term or short term packet
  if(DataLength > (ShortTermTimeAlignedBuffer.length + 4))
  {
    // Long Term:
    // Create Time aligned array  
    // Get tail
    // arrayCopy(src,srcidx, dst, dstidx, len)  
    arrayCopy(data, ZeroOffset + 4, ByteDataBuffer, 0, DataLength - ZeroOffset);
    // get head
    arrayCopy(data, 4, ByteDataBuffer, DataLength - ZeroOffset, ZeroOffset);
    // Convert all raw data to temperature in units
    for(int i=0; i<DataLength; i++)
    {
      LongTermTimeAlignedBuffer[i] = Thermostat.ConvertToTemperature(ByteDataBuffer[i]<<2);
    }
  }
  else
  {
    // Short Term:
    // Create Time aligned array  
    // Get tail
    // arrayCopy(src,srcidx, dst, dstidx, len)  
    arrayCopy(data, ZeroOffset + 4, ByteDataBuffer, 0, DataLength - ZeroOffset);
    // get head
    arrayCopy(data, 4, ByteDataBuffer, DataLength - ZeroOffset, ZeroOffset);      
    // Convert all raw data to temperature in units
    for(int i=0; i<DataLength; i++)
    {
      ShortTermTimeAlignedBuffer[i] = Thermostat.ConvertToTemperature(ByteDataBuffer[i]<<2);
    }
    // Update current Temp  
    Thermostat.temperature = ShortTermTimeAlignedBuffer[DataLength-1]; 
  }
}

/**
 * To perform any action on datagram reception, you need to implement this 
 * handler in your code. This method will be automatically called by the UDP 
 * object each time he receive a nonnull message.
 * By default, this method have just one argument (the received message as 
 * byte[] array), but in addition, two arguments (representing in order the 
 * sender IP address and his port) can be set like below.
 *
 * Receives the Acceleromter X,Y and Z axis data streamed from the MSP430
 * and CC3000.  Copies over the data to the Accel object
 */
void AccelerometerRxHandler( byte[] data, String ip, int port ) {	// <-- extended handler
  NumOfBytesReceivedCounter += data.length;
  
  if(data[0] == 0x06)
  {
    Accel.X = int(data[2]<<8) + int(data[1]);
    Accel.Y = int(data[4]<<8) + int(data[3]);
    Accel.Z = int(data[6]<<8) + int(data[5]);
    
    // will update View to ACCELEROMETER_VIEW
    AccelerometerLastUpdateMs = millis();
  }  
}

/**
 * Parse the config file to get the temperature units.
 *
 */
void ReadConfigFile () {
    XMLElement xml = new XMLElement(this, "../manifest.xml");
    XMLElement[] xmlc = xml.getChildren();
    
    XMLElement Units = xmlc[0].getChild(0);
    TemperatureUnitsString = Units.getString("type");
    
}

void DrawLogos() {
  // Draw both Logos
  image(PDEImg, PDE_IMG_X, PDE_IMG_Y); 
  image(TIImg, TI_IMG_X, TI_IMG_Y); 
 
  // See if mouse over logo
  if( (mouseY > (TI_IMG_Y)) &&
      (mouseX > (TI_IMG_X)) &&
      (mouseX < (TIImg.width + TI_IMG_X))  )
      {
        // Display link
        fill(30);
        rect(TI_IMG_X-40, TI_IMG_Y+TIImg.height, 150,30);
        fill(255);
        text(TI_LINK, TI_IMG_X-30, TI_IMG_Y+TIImg.height+ 20);
        // if pressed too then open link
        if (mousePressed == true) 
        {
          link(TI_LINK);
          while(mousePressed == true)
          {
            noLoop();
          }
          loop();
        }
      }
  if( (mouseY > (PDE_IMG_Y)) &&
      (mouseX > (PDE_IMG_X)) &&
      (mouseX < (PDEImg.width + PDE_IMG_X))  )
      {
        // Display link
        fill(30);
        rect(PDE_IMG_X, PDE_IMG_Y + PDEImg.height, 170,30);
        fill(255);
        text(PDE_LINK, PDE_IMG_X+10, PDE_IMG_Y + PDEImg.height+ 20);
        // if pressed too then open link
        if (mousePressed == true) 
        {
          link(PDE_LINK);
          while(mousePressed == true)
          {
            noLoop();
          }
          loop();
        }
      }
}
