/*
 * main.c - DataLogger_prj.
 * 
 * Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the project's author nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#include "msp430fr5739.h"
#include "adc.h" 
#include "wlan.h" 
#include "evnt_handler.h"    // callback function declaration
#include "nvmem.h"
#include "socket.h"
#include "CC3000_common.h"
#include "netapp.h"
#include "version.h"
#include <string.h>
#include <msp430.h>

#ifndef true
#define true  1
#define false 0
#endif

//
// Simple Config Prefix
//
char aucCC3000_prefix[] = {'P', 'D', 'E'};

//
// These are the main statemachine states
//
#define SEND_THERMOSTAT_STATE         2
#define GET_MESSAGES_FROM_HOST_STATE  3
#define SEND_ACCEL_XYZ_START_STATE    4
#define SEND_ACCEL_XYZ_STATE          5
#define SEND_ACCEL_XYZ_STOP_STATE     6
#define IDLE_STATE                    7

static unsigned char CC3000ConectionState;
// 
// These are the CC3000 connection status states
//
#define CC3000_INIT_STATE             1
#define CC3000_NOT_CONNECTED_STATE    2
#define CC3000_CONNECTED_STATE        3
#define CC3000_CONNECTED_PORTS_OPEN_STATE 4

//
// Various different CC3000 Status values
// returned when attempting to initialize
// and open sockets.
//
typedef enum CC3000_STATUS_T{
  CC3000_CONNECTED_PORTS_OPEN         = 0,
  CC3000_CONNECTED_NO_ADDR_ERR        = -1,
  CC3000_NO_AP_TIMEDOUT_ERR           = -2,
  CC3000_NO_AP_ERR                    = -3,
  CC3000_NO_PORTS_CLOSED_ERR          = -4
}CC3000Status_t;

//
// The Processor Clock frequency in Hz
//
#define MCLK        8000000

//
// uncomment this if you would like to use DHCP
//
//#define DHCP_EN                 1
//
// Other Wise, you can specify the IP address here
//
unsigned char g_ThisIpAddress[4] = {192, 168, 10, 3};
unsigned char g_ThisMulticastAddress[4] = {224, 0, 0, 1};

//
// Comment out if you want to force a connection to a specific AP
//
//#define SMART_CONFIG_EN         1
#define TEMPERATURE_UDP_PORT    0xC748
#define ACCELEROMETER_UDP_PORT  0xC749

//
//  MCLK / 1000 = 1ms tick. 
// Do a check that the number will fit in a 16-bit reg
//
#define TICK_RATE_1MS       MCLK / 1000
#if (TICK_RATE_1MS > 0x0000FFFF)
#error "TICK_RATE_1MS > than 16-bit"
#endif

// S1 Mapping (port4)
#define S1_BIT      BIT0
#define S1_PRESSED  ((P4IN & BIT0) ? 0:1)

// LED mapping
#define   LED_CONNECTION      0
#define   LED_TRYING_CONNECT  1
#define   LED_SIMPLE_CONFIG   2
#define   LED_TX              3
#define   LED_RX              4
//#define   LED_UNDEF           5
//#define   LED_UNDEF           6
//#define   LED_UNDEF           7

// FRAM Data logger Mem Map
#define SHORT_TERM_BUFFER_SIZE    60
#define LONG_TERM_BUFFER_SIZE     1440
// We don't use the linker here to define the 
// the data log memory in FRAM but we do use the
// linker to ensure that we reserve enough
// space.  The linker section is called "FRAM_DATA_LOG"
// Define the end to be even otherwise byte packing wont align
// CSTACK starts at 0xFE00 so we subtract to just to be safe.
// Each buffer has 2 16-bit headers reserved for the Length and the 
// index of the T=0 data
#define LONG_TERM_END_ADD         (0xF0A0)
// We find the start address based off the end address
#define LONG_TERM_START_ADD       (LONG_TERM_END_ADD - LONG_TERM_BUFFER_SIZE)
// We reserve two more 16-bit values to store the Length and T=0 position
// This information is sent in the WiFi packet as a header
#define LONG_TERM_HEADER_ADD      (LONG_TERM_START_ADD - (2*sizeof(short)))

// Now we repeat the definitions for the short term buffer.
#define SHORT_TERM_END_ADD        (LONG_TERM_HEADER_ADD)
#define SHORT_TERM_START_ADD      (SHORT_TERM_END_ADD - SHORT_TERM_BUFFER_SIZE)
#define SHORT_TERM_HEADER_ADD     (SHORT_TERM_START_ADD - (2*sizeof(short)))

// Stores the pointers to the above buffers
#define FRAM_PTR_SHORT_TERM_LOC   (SHORT_TERM_HEADER_ADD - sizeof(short))
#define FRAM_PTR_LONG_TERM_LOC    (FRAM_PTR_SHORT_TERM_LOC - sizeof(short))

extern AdcMeasurement_t AdcDevices;

static unsigned short *g_PtrShortTermBuffer, *g_PtrLongTermBuffer;
static unsigned char g_TimeToTxFlag = true;
static unsigned long g_ulSocket;
static unsigned long g_ulSmartConfigFinished;
static unsigned char g_ucHaveIpAddress = false;
static unsigned char g_DeletePolicy = false;

//*****************************************************************************
//
//! Delay100ms
//!
//! \param  none
//!
//! \return none
//!
//! \brief  Performs a long delay of 100ms based on MCLK
//!				
//
//*****************************************************************************
void Delay100ms()
{
  __delay_cycles(MCLK/10);
  __no_operation();
}
//*****************************************************************************
//
//! SystemInit
//!
//!  \param  None
//!
//!  \return none
//!
//!  \brief  Initializes system
//
//*****************************************************************************
void SystemInit(void)
{
	// Startup clock system in max. DCO setting ~8MHz
	// This value is closer to 10MHz on untrimmed parts  
	CSCTL0_H = 0xA5;                          // Unlock register
	CSCTL1 |= DCOFSEL0 + DCOFSEL1;            // Set max. DCO setting
	CSCTL2 = SELA__VLOCLK + SELS__DCOCLK + SELM__DCOCLK;
	CSCTL3 = DIVA_0 + DIVS_0 + DIVM_0;        // set all dividers
	CSCTL0_H = 0x01;                          // Lock Register
	
	// Turn off temp. 
	REFCTL0 |= REFTCOFF; 
	REFCTL0 &= ~REFON;  
	
	// Enable switches
	// (S1)P4.0 is configured as a switch 
	// (S2)P4.1  is reserved for the CC3000
	P4OUT |= S1_BIT;                     // Configure pullup resistor  
	P4DIR &= ~(S1_BIT);                  // Direction = input
	P4REN |= S1_BIT;                     // Enable pullup resistor
	
	// P4.1 - WLAN enable full DS
	P4OUT &= ~BIT1;
	P4DIR |= BIT1;
	P4SEL1 &= ~BIT1; 
	P4SEL0 &= ~BIT1;
	
	// Enable LEDs
	P3OUT &= ~(BIT6+BIT7+BIT5+BIT4);
	P3DIR |= BIT6+BIT7+BIT5+BIT4;
	PJOUT &= ~(BIT0+BIT1+BIT2+BIT3);
	PJDIR |= BIT0 +BIT1+BIT2+BIT3;
	
	// Terminate Unused GPIOs
	// P1.0 - P1.6 is unused
	P1OUT &= ~(BIT0 + BIT1 + BIT2 + BIT3 + BIT5 + BIT6 + BIT7);   
	P1DIR &= ~(BIT0 + BIT1 + BIT2 + BIT3 + BIT5 + BIT6 + BIT7); 
	P1REN |= (BIT0 + BIT1 + BIT2 + BIT3 + BIT5 + BIT6 + BIT7);   
	
	// P1.4 is used as input from NTC voltage divider
	// Set it to output low
	P1OUT &= ~BIT4;      
	P1DIR |= BIT4; 
  
	// Configure the SPI CS to be on P1.3
	P1OUT |= BIT3;
	P1DIR |= BIT3;
	P1SEL1 &= ~BIT3; 
	P1SEL0 &= ~BIT3;
	
	// P2.2 - P2.6 is unused
	P2OUT &= ~(BIT2 + BIT4 + BIT5 + BIT6);    
	P2DIR &= ~(BIT2 + BIT4 + BIT5 + BIT6); 
	P2REN |= (BIT2 + BIT4 + BIT5 + BIT6);   
	
	// Configure SPI IRQ line on P2.3
	P2DIR  &= (~BIT3);	
	P2SEL1 &= ~BIT3; 
	P2SEL0 &= ~BIT3;
	// Clear Flag
	P2IFG  &= ~BIT3;
	
	// Configure detection of the tripping of Magnetic Sensor on P2.2
	P2DIR  &= (~BIT2);	
	P2SEL1 &= ~BIT2; 
	P2SEL0 &= ~BIT2;
	// Clear Flag
	P2IFG  &= ~BIT2;
	
	// Configure UART pins P2.0 & P2.1
	P2SEL1 |= BIT0 + BIT1;
	P2SEL0 &= ~(BIT0 + BIT1);
	
	// P2.7 is used to power the voltage divider for the NTC thermistor
	P2OUT &= ~BIT7;     
	P2DIR |= BIT7; 
	
	// P3.0,P3.1 and P3.2 are accelerometer inputs
	P3OUT &= ~(BIT0 + BIT1 + BIT2);   
	P3DIR &= ~(BIT0 + BIT1 + BIT2); 
	P3REN |= BIT0 + BIT1 + BIT2;
	
	// PJ.0,1,2,3 are used as LEDs
	// crystal pins for XT1 are unused
	PJOUT &= ~(BIT4+BIT5);  
	PJDIR &= ~(BIT4+BIT5);
	PJREN |= BIT4 + BIT5;  
}

//*****************************************************************************
//
//! PeriodicTimerInit
//!
//! \param  none
//!
//! \return none
//!
//! \brief  Sets up TimerA0 for 1ms periodic timer intervals to take
//!         ADC sequence samples. 
//!				
//
//*****************************************************************************
void PeriodicTimerInit(void)
{
  TA0CCTL0 = CCIE;                          // TACCR0 interrupt enabled
  
  TA0CCR0 = TICK_RATE_1MS;
  TA0CTL = TASSEL_2 + MC_2;                 // SMCLK, continuous mode
}

//*****************************************************************************
//
//! LedOn
//!
//! \param  Led Index
//!
//! \return none
//!
//! \brief  Sets Led
//!				
//
//*****************************************************************************
void LedOn(unsigned char led)
{   
  if (led <4)
  {
    PJOUT |= 0x01<<led;        
  }
  else
  {    
    P3OUT |= 0x01<<led; 
  }           
}
void LedOff(unsigned char led)
{   
  if (led <4)
  {
    PJOUT &= ~(0x01<<led);        
  }
  else
  {    
    P3OUT &= ~(0x01<<led);
  }           
}
void LedToggle(unsigned char led)
{   
  if (led <4)
  {
    if(PJOUT & (0x01<<led) )
    {
      PJOUT &= ~(0x01<<led);   
    }
    else
    {
      PJOUT |= 0x01<<led;  
    }
  }
  else
  {    
    if(P3OUT & (0x01<<led) )
    {
      P3OUT &= ~(0x01<<led);   
    }
    else
    {
      P3OUT |= 0x01<<led;  
    }
  }           
}

void SaveAndResetSystem(void)
{
  
  // Rest CPU by writing to WD with illegal password.
  //WDTCTL = (0xFF00u) + WDTHOLD;  
}

//*****************************************************************************
//
//! reportAccelXyz
//!
//!  \param  None
//!
//!  \return none
//!
//!  \brief  Sends the Acceleromter XYZ values to the PC
//
//*****************************************************************************
void reportAccelXyz(void)
{
  sockaddr tSocketAddr;
  unsigned short usTemp;
  unsigned char ucTX_Buffer[7];
  
    LedOn(LED_TX);
    { 	
      // Transmit 7 Bytes
      // Byte 1 is header defining the packet type
      ucTX_Buffer[0] = 0x06; 
      usTemp = AdcDevices.AccelX;
      ucTX_Buffer[1] = usTemp;
      ucTX_Buffer[2] = usTemp>>8;
      
      usTemp = AdcDevices.AccelY;
      ucTX_Buffer[3] = usTemp;
      ucTX_Buffer[4] = usTemp>>8;
      
      usTemp = AdcDevices.AccelZ;
      ucTX_Buffer[5] = usTemp;
      ucTX_Buffer[6] = usTemp>>8;
      // Write to UDP port    
      tSocketAddr.sa_family = AF_INET;
      //
      // port and IP address
      //
      tSocketAddr.sa_data[0] = (unsigned char)(ACCELEROMETER_UDP_PORT>>8);
      tSocketAddr.sa_data[1] = (unsigned char)ACCELEROMETER_UDP_PORT;
      
      memcpy(&tSocketAddr.sa_data[2], g_ThisMulticastAddress, 4);
      
      sendto(g_ulSocket, ucTX_Buffer, sizeof(ucTX_Buffer), 0, &tSocketAddr,
             sizeof(sockaddr));
    } 
    LedOff(LED_TX);
}
//*****************************************************************************
//
//! reportTemperature
//!
//!  \param  None
//!
//!  \return none
//!
//!  \brief  Sends the temperature to the PC
//
//*****************************************************************************
void reportTemperature(void)
{
  sockaddr tSocketAddr;
  unsigned char *pucDataPtr;
  
    LedOn(LED_TX);
    { 	
      //first set up the UDP socket and multicast address.
      tSocketAddr.sa_family = AF_INET;
      //
      // port and IP address
      //
      tSocketAddr.sa_data[0] = (unsigned char)(TEMPERATURE_UDP_PORT>>8);
      tSocketAddr.sa_data[1] = (unsigned char)TEMPERATURE_UDP_PORT;
      
      memcpy(&tSocketAddr.sa_data[2], g_ThisMulticastAddress, 4);
      
      // First transmit the short term buffer and then the long term buffer.
      // We send the data as-is in FRAM.  We reserved 2 16-bit values at the 
      // head of the buffer so that we can write the Length and the pointer 
      // position at the begging.
      
      // Get pointer to header in FRAM
      pucDataPtr = (unsigned char*)SHORT_TERM_HEADER_ADD;
      // get the data length
      *(unsigned short *)pucDataPtr = SHORT_TERM_BUFFER_SIZE;
      // advance pointer 
      pucDataPtr += 2;
      // get the position offset of the pointer to data.  The PC will then
      // time align the data since it has more resources for processing.
      *(unsigned short *)pucDataPtr = (unsigned short)*g_PtrShortTermBuffer -
                                   (unsigned short)(SHORT_TERM_START_ADD);
      // quick check that we haven't exceeded the CC3000 Tx buffer size
      if(*(unsigned short *)pucDataPtr > 1700/*SPI_BUFFER_SIZE*/ )
      {
        while(1);;
      }
        
      // Write to UDP port    
      // add 4 to the SHORT_TERM_BUFFER_SIZE to compensate for the 2 16bit 
      // header values
      sendto(g_ulSocket, (const void *)SHORT_TERM_HEADER_ADD,
             SHORT_TERM_BUFFER_SIZE + 4, 0, &tSocketAddr, sizeof(sockaddr));
      
      //
      // Now send the long term buffer.
      //
      // Get pointer to header in FRAM
      pucDataPtr = (unsigned char*)LONG_TERM_HEADER_ADD;
      // get the data length
      *(unsigned short *)pucDataPtr = LONG_TERM_BUFFER_SIZE;
      // advance pointer 
      pucDataPtr += 2;
      // get the position offset of the pointer to data.  The PC will then
      // time align the data since it has more resources.
      *(unsigned short *)pucDataPtr = (unsigned short)*g_PtrLongTermBuffer -
                                   (unsigned short)(LONG_TERM_START_ADD);
      // quick check that we haven't exceeded the CC3000 Tx buffer size
      if(*(unsigned short *)pucDataPtr > 1700/*SPI_BUFFER_SIZE*/ )
      {
        while(1);;
      }
        
      // Write to UDP port    
      // add 4 to the SHORT_TERM_BUFFER_SIZE to compensate for the 2 16bit 
      // header values
      sendto(g_ulSocket, (const void *)LONG_TERM_HEADER_ADD,
             LONG_TERM_BUFFER_SIZE + 4, 0, &tSocketAddr, sizeof(sockaddr));
    } 
    LedOff(LED_TX);
}

//*****************************************************************************
//
//! This function: init_spi
//!
//!  \param  none
//!
//!  \return none
//!
//!  \brief  initializes an SPI interface
//
//*****************************************************************************
int init_spi(void)
{
	// Select the SPI lines: MISO/MOSI on P1.6,7 CLK on P2.2
	P1SEL1 |= (BIT6 + BIT7);
	P1SEL0 &= (~(BIT6 + BIT7));

	P2SEL1 |= (BIT2);
	P2SEL0 &= ~BIT2;
	
	UCB0CTLW0 |= UCSWRST;                 // **Put state machine in reset**
	UCB0CTLW0 |= (UCMST+UCSYNC+UCMSB);   	// 3-pin, 8-bit SPI master
	                                      // Clock polarity high, MSB
	UCB0CTLW0 |= UCSSEL__SMCLK;           // UCSSEL__SMCLK
	UCB0BR0 = 2;                          // /8 of SMCLK = 24Mhz (3mhz)
	UCB0BR1 = 0;                          //

	UCB0CTLW0 &= ~UCSWRST;                // **Initialize USCI state machine**
    
  return(ESUCCESS);
}


//*****************************************************************************
//
//! sendDriverPatch
//!
//! \param  pointer to the length
//!
//! \return none
//!
//! \brief  The function returns a pointer to the driver patch
//
//*****************************************************************************
char *sendDriverPatch(unsigned long *Length)
{
	*Length = 0;
	return NULL;
}


//*****************************************************************************
//
//! sendBootLoaderPatch
//!
//! \param  pointer to the length
//!
//! \return none
//!
//! \brief  The function returns a pointer to the boot loader patch: 
//!				since there is no patch yet - it returns 0
//
//*****************************************************************************
char *sendBootLoaderPatch(unsigned long *Length)
{
	*Length = 0;
	return NULL;
}

//*****************************************************************************
//
//! sendWLFWPatch
//!
//! \param  pointer to the length
//!
//! \return none
//!
//! \brief  The function returns a pointer to the FW patch
//
//*****************************************************************************

char *sendWLFWPatch(unsigned long *Length)
{
	*Length = 0;
	return NULL;
}

//*****************************************************************************
//
//! ReadWlanInterruptPin
//!
//! \param  none
//!
//! \return none
//!
//! \brief  return wlan interrup pin
//
//*****************************************************************************

long ReadWlanInterruptPin(void)
{
	return (P2IN & BIT3);
}

//*****************************************************************************
//
//! Enable waln IrQ pin
//!
//! \param  none
//!
//! \return none
//!
//! \brief  None
//
//*****************************************************************************


void WlanInterruptEnable()
{
	__bis_SR_register(GIE);
	P2IES |= BIT3;
	P2IE |= BIT3;
}

//*****************************************************************************
//
//! Disable waln IrQ pin
//!
//! \param  none
//!
//! \return none
//!
//! \brief  Nonr
//
//*****************************************************************************


void WlanInterruptDisable()
{
	P2IE &= ~BIT3;
}


//*****************************************************************************
//
//! WriteWlanPin
//!
//! \param  new val
//!
//! \return none
//!
//! \brief  void
//
//*****************************************************************************

void WriteWlanPin( unsigned char val )
{
	if (val)
	{
		P4OUT |= BIT1;	
	}
	else
	{
		P4OUT &= ~BIT1;
	}

}


//*****************************************************************************
//
//! CC3000_UsynchCallback
//!
//! \param  Event type
//!
//! \return none
//!
//! \brief  The function handles asynchronous events that come from CC3000 device 
//!		  and operates a LED4 to have an on-board indication
//
//*****************************************************************************

void CC3000_UsynchCallback(long lEventType, char * data, unsigned char length)
{
	if (lEventType == HCI_EVNT_WLAN_ASYNC_SIMPLE_CONFIG_DONE)
	{
		g_ulSmartConfigFinished = 1;
	}

	if (lEventType == HCI_EVNT_WLAN_UNSOL_CONNECT)
	{
		CC3000ConectionState = CC3000_CONNECTED_STATE;

		//
		// Turn on the LED1
		//
		LedOn(LED_CONNECTION);
#ifndef DHCP_EN
        g_ucHaveIpAddress = true;
#endif
	}

	if (lEventType == HCI_EVNT_WLAN_UNSOL_DISCONNECT)
	{		
		CC3000ConectionState = CC3000_NOT_CONNECTED_STATE;

		//
		// Turn off the LED1
		//
		LedOff(LED_CONNECTION); 
    
        // Connection to AP unintentionally closed. Reset
        SaveAndResetSystem();
	}
  
  if(lEventType == HCI_EVNT_WLAN_UNSOL_DHCP)
  {
        //
        // Should have an IP address now. Can send data.
        //
        g_ucHaveIpAddress = true;
        //
        // Get our assigned address from the passed data
        //
        g_ThisIpAddress[0] = data[3];
        g_ThisIpAddress[1] = data[2];
        g_ThisIpAddress[2] = data[1];
        g_ThisIpAddress[3] = data[0];
  }
}

//*****************************************************************************
//
//! initCC3000Driver
//!
//!  \param  None
//!
//!  \return none
//!
//!  \brief  The function initializes a CC3000 device and triggers it to start operation
//
//*****************************************************************************
int initCC3000Driver(void)
{
	//
    //init all layers
    //
    init_spi();
  
	//
	// WLAN On API Implementation
	//
	wlan_init( CC3000_UsynchCallback, sendWLFWPatch, sendDriverPatch,
             sendBootLoaderPatch, ReadWlanInterruptPin, WlanInterruptEnable,
             WlanInterruptDisable, WriteWlanPin);

	//
	// add just over a 1 second delay to ensure that the slow clock is stable
	//
	__delay_cycles(2410000);

	CC3000ConectionState = CC3000_NOT_CONNECTED_STATE;
  
	//
	// Trigger a WLAN device.  Set true because there are patches available
	//
	wlan_start(false);
  
	//
	// Mask out all non-required events from CC3000
	//
#ifdef DHCP_EN
    //
    // Setup DHCP.  Although this should be unnecessary as DHCP is the default
    //
    unsigned char pucSubnetMask[4], pucIP_Addr[4], pucIP_DefaultGWAddr[4],
            pucDNS[4];
    memset(pucSubnetMask,       0, sizeof(pucSubnetMask));
    memset(pucIP_Addr,          0, sizeof(pucIP_Addr));
    memset(pucIP_DefaultGWAddr, 0, sizeof(pucIP_DefaultGWAddr));
    memset(pucDNS,              0, sizeof(pucDNS));
    
    netapp_dhcp((unsigned long *) pucIP_Addr, (unsigned long *) pucSubnetMask,
            (unsigned long *) pucIP_DefaultGWAddr, (unsigned long *) pucDNS);
    wlan_set_event_mask(HCI_EVNT_WLAN_KEEPALIVE |
                        HCI_EVNT_WLAN_UNSOL_INIT |
                        HCI_EVNT_WLAN_ASYNC_PING_REPORT);
#else
    //
    // Setup DHCP
    //
	  unsigned char pucSubnetMask[4], pucIP_Addr[4], pucIP_DefaultGWAddr[4],
			    pucDNS[4];
    memset(pucSubnetMask,       0xFF, sizeof(pucSubnetMask));
    memcpy(pucIP_Addr, g_ThisIpAddress, sizeof(g_ThisIpAddress));
        
    pucIP_DefaultGWAddr[0] = g_ThisIpAddress[0];
    pucIP_DefaultGWAddr[1] = g_ThisIpAddress[1];
    pucIP_DefaultGWAddr[2] = g_ThisIpAddress[2];
    pucIP_DefaultGWAddr[3] = 1;
	  
    memset(pucDNS,              0, sizeof(pucDNS));
    netapp_dhcp((unsigned long *) pucIP_Addr, (unsigned long *) pucSubnetMask,
        (unsigned long *) pucIP_DefaultGWAddr, (unsigned long *) pucDNS);
    wlan_set_event_mask(HCI_EVNT_WLAN_KEEPALIVE  |
                        HCI_EVNT_WLAN_UNSOL_INIT |
                        HCI_EVNT_WLAN_UNSOL_DHCP |
                        HCI_EVNT_WLAN_ASYNC_PING_REPORT);
#endif
  return(0);
}

//*****************************************************************************
//
//! FramDataLogInit
//!
//!  \param  None
//!
//!  \return none
//!
//!  \brief  The function loads the pointers to the short-term and
//!          long-term data log buffers.  The pointers are stored in
//!          FRAM so that the data logging will continue where it left
//!          off after power up.  The Pointers are then checked to
//!          verify that they are pointing to a valid region and if
//!          not reset them.  This will happen during the first time
//!          the device is powered up.
//
//******************************************************************
void FramDataLogInit(void)
{
  // Get the Ptrs to Short and long Term Buffers
  g_PtrShortTermBuffer = (unsigned short*)FRAM_PTR_SHORT_TERM_LOC;
  g_PtrLongTermBuffer =  (unsigned short*)FRAM_PTR_LONG_TERM_LOC;
  
  // Check if the pointers are valid.  If not then we are probably runnging for 
  // the first time.
  if( (*g_PtrShortTermBuffer > SHORT_TERM_END_ADD) || 
      (*g_PtrShortTermBuffer < SHORT_TERM_START_ADD) || 
      (*g_PtrLongTermBuffer  > LONG_TERM_END_ADD) || 
      (*g_PtrLongTermBuffer  < LONG_TERM_START_ADD) )
  {
    *g_PtrShortTermBuffer = SHORT_TERM_START_ADD;
    *g_PtrLongTermBuffer =  LONG_TERM_START_ADD;
  }
}

//*****************************************************************************
//
//! FramDataLog
//!
//!  \param  None
//!
//!  \return none
//!
//!  \brief  The function will data log every sample into the short-term buffer
//!          and every 60th sample into the long term buffer.  It is assumed that 
//!          this will be called once every second and since the short term buffer
//!          is 60 bytes long then will keep a short term history of the temperature
//!          every second for 1 minute.  Also, the long-term buffer is 1440
//!          bytes long and represents 1 sample per minute for 24hours.  
//!          After 24hrs the buffer wraps around.
//
//*****************************************************************************
void FramDataLog(unsigned char Temperature)
{  
  static unsigned char  uc1SecondTicks = 0;
  unsigned char ucOldestShortTermData = 0;  
  
  {    
    // First fetch the oldest data that will be over written
    ucOldestShortTermData = (*(unsigned char *)(*g_PtrShortTermBuffer));
    // then Log Data in short Term buffer
    (*(unsigned char *)(*g_PtrShortTermBuffer)) = Temperature;
    // Move Pointer to next available spot
    (*g_PtrShortTermBuffer)++;
    if(*g_PtrShortTermBuffer >= SHORT_TERM_END_ADD)
    {
      (*g_PtrShortTermBuffer) = SHORT_TERM_START_ADD;
    }
    
    uc1SecondTicks++;
    // 1/minute
    if(uc1SecondTicks >= 60)
    {
      // Log Data in Long Term buffer
      (*(unsigned char *)(*g_PtrLongTermBuffer)) =  ucOldestShortTermData;
    
      // Move Pointer to next available spot
      (*g_PtrLongTermBuffer)++;
      if(*g_PtrLongTermBuffer >= LONG_TERM_END_ADD)
      {
        (*g_PtrLongTermBuffer) = LONG_TERM_START_ADD;
      }
      uc1SecondTicks = 0;
    }
  }
}
//*****************************************************************************
//
//! StartSmartConfig
//!
//!  \param  None
//!
//!  \return none
//!
//!  \brief  The function triggers a smart configuration process on CC3000.
//!			it exists upon completion of the process
//
//*****************************************************************************
void StartSmartConfig(void)
{
    g_ulSmartConfigFinished = 0;
    //
    // Reset all the previous configuration
    //
    wlan_ioctl_set_connection_policy(0,0,0);	
    
    //
    // Trigger the Smart Config process
    //
    
    // Start blinking LED2 during Smart Configuration process
    LedOff(LED_SIMPLE_CONFIG);
    
    wlan_first_time_config_set_prefix(aucCC3000_prefix);
         
    //
    // Start the Smart Config process
    //
    wlan_first_time_config_start();
    
    //
    // Wait for simple config finished
    // Set a 1 minute timeout to be safe 
    //
    unsigned char ucTimeOut = 60;
    while (g_ulSmartConfigFinished == 0 &&
           ucTimeOut)
    {
      __delay_cycles(6000000);
    
      LedOn(LED_SIMPLE_CONFIG);
      
      __delay_cycles(6000000);
                  
      LedOff(LED_SIMPLE_CONFIG);
      
      ucTimeOut--;
    }
    
    LedOff(LED_SIMPLE_CONFIG);
    
    //
    // Configure to connect automatically to the AP retrieved in the 
    // simple config process
    //
    wlan_ioctl_set_connection_policy(0, 0, 1);
    
    //
    // reset the CC3000
    // 
    wlan_stop();
    
    Delay100ms();
    
    wlan_start(false);

    //
    // Mask out all non-required events
    //
#ifdef DHCP_EN
    wlan_set_event_mask(HCI_EVNT_WLAN_KEEPALIVE  |
                    HCI_EVNT_WLAN_UNSOL_INIT |
                    HCI_EVNT_WLAN_ASYNC_PING_REPORT);
#else    
    wlan_set_event_mask(HCI_EVNT_WLAN_KEEPALIVE  |
                        HCI_EVNT_WLAN_UNSOL_INIT |
                        HCI_EVNT_WLAN_UNSOL_DHCP |
                        HCI_EVNT_WLAN_ASYNC_PING_REPORT);
#endif
}

//*****************************************************************************
//
//! CC3000EnableAndOpenPort
//!
//!  \param  None
//!
//!  \return none
//!
//!  \brief  This function initializes the CC3000 and establishes 
//!			a connection to an AP, then opens a port.  The status of
//!     this function is returned as a CC3000Status_t enum
//
//*****************************************************************************
CC3000Status_t CC3000EnableAndOpenPort( void )
{
  CC3000Status_t Status = CC3000_NO_AP_ERR;
  unsigned char i;
  //
  //  Start the CC3000
  //
  initCC3000Driver();    
  //
  // If button is pressed then the user wants to delete the policy 
  // so they can connect to a different AP most likely.
  //
  if(g_DeletePolicy == true)
  {
    //
    // Reset all the previous configuration
    //
    wlan_ioctl_set_connection_policy(0,0,0);	
    wlan_ioctl_del_profile(0);
    wlan_ioctl_del_profile(1);
    wlan_ioctl_del_profile(2);
    //
    // reset the CC3000
    // 
    wlan_stop();      
    initCC3000Driver();
    
    g_DeletePolicy = false;
  }
  else
  {  
    //
    // Configure to connect automatically to the AP retrieved in the 
    // simple config process
    //
    wlan_ioctl_set_connection_policy(0, 0, 1);
  }
  
#ifdef SMART_CONFIG_EN      
    //
    // Wait 12 seconds to see if we connect
    //    
    for(i = 0; (i < 60) &&
       (CC3000ConectionState == CC3000_NOT_CONNECTED_STATE); i++)
    {
      Delay100ms();
      LedOn(LED_TRYING_CONNECT);
      Delay100ms();
      LedOff(LED_TRYING_CONNECT);
    }
#endif
  //
  // test connection status
  // if we didn't get a connection to an AP then we can try the 
  // smart first time config which listens for the secret.
  if(CC3000ConectionState == CC3000_NOT_CONNECTED_STATE)
  {
    //
    // Try Connect to the AP using smart config
    //
#ifdef SMART_CONFIG_EN
#if (defined(SECURITY_WEP) || defined(SECURITY_WPA) || defined(SECURITY_WPA2))
#error "Error: Smart Config cannot be built with wlan_connect method.  Please Choose only one method"
#endif
    StartSmartConfig();     
#else
        // or, direct connect
#if   defined(SECURITY_WEP)
		wlan_connect(1, "TP-LINK", 0x07,
					NULL, "password", sizeof("password"));
#elif  defined(SECURITY_WPA)
		wlan_connect(2, "TP-LINK", 0x07,
					NULL, "password", sizeof("password"));
#elif  defined(SECURITY_WPA2)
		wlan_connect(3, "TP-LINK", 0x07,
					NULL, "password", sizeof("password"));
#else // no security
		wlan_connect(0, "TP-LINK", 0x07, NULL, NULL, 0);
#endif          
#endif      
    //
    // Wait 3 seconds to see if we connect this time
    //    
    for(i = 0; (i < 15) &&
       (CC3000ConectionState == CC3000_NOT_CONNECTED_STATE); i++)
    {
      Delay100ms();
      LedOn(LED_TRYING_CONNECT);
      Delay100ms();
      LedOff(LED_TRYING_CONNECT);
    }
  }
  
  
  //
  // Now test we made a connection to the AP
  //
  if(CC3000ConectionState == CC3000_CONNECTED_STATE)
  {
    // wait for an address
#ifdef DHCP_EN
    // wait for a DHCP addr (60 * 0.2s = 12seconds max wait)
    for(i = 0; (i < 60) &&
     (g_ucHaveIpAddress == false); i++)
    {
      Delay100ms();
      LedOn(LED_TRYING_CONNECT);
      Delay100ms();
      LedOff(LED_TRYING_CONNECT);
    }
#endif
    
    if( g_ucHaveIpAddress == true)
    {
      //
      // If interested you could get the SSID and ip address now
      // using netapp_ipconfig
      //
      /*static tNetappIpconfigRetArgs ipconfig;
      netapp_ipconfig( &ipconfig);*/
      //
      // Also, If DHCP is used then we also need to verify that we have a valid 
      // address. If so then we can now Open UDP socket.
      //
      g_ulSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP); 
      if((signed long)g_ulSocket < 0)
      {
        Status = CC3000_NO_PORTS_CLOSED_ERR;  
      }
      else
      {
        Status = CC3000_CONNECTED_PORTS_OPEN;  
      }
    }
    else
    {
      Status = CC3000_CONNECTED_NO_ADDR_ERR;
    }
  }
  else
  {
    // No connection to AP. return error
    Status = CC3000_NO_AP_TIMEDOUT_ERR;
  }
  
  // Now check the connection status before we leave.  If we were not succesful
  // we should just power down the AP to conserve power unless we made the 
  // connection to the AP but do not have a valid Address yet.  If this is
  // the case we don't power down as we want to keep the CC3000 alive long 
  // enough to get the DHCP address.
  if( Status != CC3000_CONNECTED_PORTS_OPEN)
  {
    //
    // We were unsuccesful. Power down CC3000
    //
    WriteWlanPin( WLAN_DISABLE );
    CC3000ConectionState = CC3000_NOT_CONNECTED_STATE;
    LedOff(LED_CONNECTION);
  }
  return(Status);
}
    
void main(void)
{
  static unsigned char ucS1Pressed = false;
  unsigned char i;

  *g_PtrShortTermBuffer = *g_PtrLongTermBuffer = 0;
   g_TimeToTxFlag = true;
  g_ulSocket = 0;
  g_ulSmartConfigFinished = 0;
  g_ucHaveIpAddress = false;
  g_DeletePolicy = false;
  //
  //  Stop WDT
  //
#ifdef __CCS__
	 WDTCTL = WDTPW + WDTHOLD;
#elif __IAR_SYSTEMS_ICC__
	 asm("mov.w   #0x5a80,&0x015C; ");
#endif
  //
  //  Board Initialization start
  //
  SystemInit();
  
  //
  // If Button is pressed on power up then flag the CC3000
  // processes to delete the policy the first time through.
  //
  if(S1_PRESSED)
  {
    g_DeletePolicy = true;
  }
  //
  //  ADC Initialization for Thermistor and Accelerometer
  //
  AdcInit();
  
  //
  //  FRAM data logger Initialization. Gets pointer to last log
  //
  FramDataLogInit();
  //
  //  Timer Initialization for periodic Thermistor and Accelerometer
  //  sampling
  //
  PeriodicTimerInit();
  //
  // Enable Global interrupts
  //
  __enable_interrupt();

  unsigned char state = SEND_THERMOSTAT_STATE;
  CC3000Status_t CC3000Status;
  
  while(1)  
  {
    // Variable initialization
    switch(state)
    {        
      case SEND_THERMOSTAT_STATE:        
        if(g_TimeToTxFlag == true)
        {
          // CC3000 should be powered off
          // First we have to initialize the CC3000
          // and then if connected we can send the data
          CC3000Status = CC3000EnableAndOpenPort();
          if( CC3000_CONNECTED_PORTS_OPEN == CC3000Status )
          {
                        
            reportTemperature();
            // Finished Sending. We can turn the CC3000 off now
            // to save power.  But before we do wait for a couple seconds to 
            // ensure all the data has been sent.  Another approach would be
            // to split the packet up into 4 packets.  The CC3000 will send the
            // packets immediately if the buffers are full.
            for(i=0; i<20; i++)
            {
              Delay100ms();
            }
            WriteWlanPin(WLAN_DISABLE); 
            g_ucHaveIpAddress = false;
            CC3000ConectionState = CC3000_NOT_CONNECTED_STATE;
            LedOff(LED_CONNECTION);
            state = IDLE_STATE;
          }
          else
          {
            state = IDLE_STATE;
          }
          g_TimeToTxFlag = false;
        }
      break;    
    case SEND_ACCEL_XYZ_START_STATE:
      // When the button is first pressed we start here and enable the CC3000
      // open the port and send the first pkt
      CC3000Status = CC3000EnableAndOpenPort();
      if( CC3000_CONNECTED_PORTS_OPEN == CC3000Status )
      {
        Delay100ms();
        Delay100ms();
        Delay100ms();
        Delay100ms();
        Delay100ms();
        reportAccelXyz();
        state = IDLE_STATE;
      }
      else
      {
        state = IDLE_STATE;
      }
      break;
    case SEND_ACCEL_XYZ_STATE:
      // Port is already open so we can just continue to send data      
      if( CC3000_CONNECTED_PORTS_OPEN == CC3000Status )
      {
        //Delay100ms();
        reportAccelXyz();
        if(g_TimeToTxFlag == true)
        {
          reportTemperature();
          g_TimeToTxFlag = false;
        }
        state = IDLE_STATE;
      }
      else
      {
        state = IDLE_STATE;
      }
      break;
    case SEND_ACCEL_XYZ_STOP_STATE:
      // Button must have been released so now we need to 
      // Turn off the CC3000
      WriteWlanPin(WLAN_DISABLE);            
      g_ucHaveIpAddress = false;
      CC3000ConectionState = CC3000_NOT_CONNECTED_STATE;
      LedOff(LED_CONNECTION);
      state = IDLE_STATE;
      break;
    case GET_MESSAGES_FROM_HOST_STATE: 
      // This application does not receive
      state = IDLE_STATE;
      break;
    case IDLE_STATE:
    default:      
      if( (CC3000_CONNECTED_STATE == CC3000ConectionState) ||
          (CC3000_CONNECTED_PORTS_OPEN_STATE == CC3000ConectionState) )
      {           
        hci_unsolicited_event_handler();
      }
      
      //
      // Check the state of S1.  S1 is used to 
      // control the stream of Accelerometer data
      // to the PC.  While the Button is asserted
      // The Acceleromter data is streamed.  The temperature
      // Data continues to be sent at 10s intervals as well
      // Once the switch is released the statemachine reverts
      // back to just sending the temperature data.
      //
      if(S1_PRESSED)
      {
        // See if the swwitch was just pressed.
        // If so, then we need to first enable the CC3000
        // and send the first packet.
        if(ucS1Pressed == false)
        {
          ucS1Pressed = true;
          state = SEND_ACCEL_XYZ_START_STATE;
        }
        else
        {
          // Continue to send.  CC3000 is already powered up
          state = SEND_ACCEL_XYZ_STATE;
        }          
      }
      else
      {
        // See if the switch was just released
        if(ucS1Pressed == true)
        {
          ucS1Pressed = false;
          // Done sending. Turn off the CC3000 to conserve power.
          state = SEND_ACCEL_XYZ_STOP_STATE;
        }
        else
        {
          // Normal low power data logging.  Check to see
          // if is time to send the temperature data.
          if(g_TimeToTxFlag == true)
          {
            state = SEND_THERMOSTAT_STATE;
          }
          else
          {
            state = IDLE_STATE;
          }
        }
      }
      break;
    }
  }
  
}
 
// Interrupt Service Routines
/**********************************************************************//**
 * @brief  Timer A0 ISR for MODE2, Slow FRAM writes, 40ms timer
 * 
 * @param  none 
 *  
 * @return none
 *************************************************************************/
#pragma vector = TIMER0_A0_VECTOR
__interrupt void Timer_A (void)
{  
  static unsigned short us1MilliTicks = 0;
  static unsigned char  uc1SecondTicks = 0;
  
  TA0CCR0 += TICK_RATE_1MS;
  AdcUpdate();
  // log 8 bit data
  us1MilliTicks++;
  // 1 per second
  if(us1MilliTicks >= 1000)
  {
    FramDataLog((unsigned char)AdcDevices.Temperature>>2);
    us1MilliTicks = 0;
    uc1SecondTicks++;
    
    // 1 per 10 seconds
    if(uc1SecondTicks >= 10)
    {
      g_TimeToTxFlag = true;
      uc1SecondTicks = 0;
    }
  }  
}

